
/******************************************************************************
* MODULE     : vector.hpp
* DESCRIPTION: Fixed size vectors
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_VECTOR_HPP
#define __MML_VECTOR_HPP
#include "base.hpp"
namespace mml {
#define TMPL template<typename T, typename K>
TMPL class vector;

/******************************************************************************
* Different types of supported vectors
******************************************************************************/

class vector_dense {
public:
  static const int def_len= 0;
};

/*
class vector_sparse {
  static const int def_len= 0;
};
*/

template<int n>
class vector_fixed {
public:
  static const int def_len= n;
};

/******************************************************************************
* Dense vectors
******************************************************************************/

template<typename T, typename K=vector_dense>
class vector_rep REP_STRUCT {
public:
  T*  a;
  int n;
public:
  inline vector_rep (T* a2, int n2): a (a2), n (n2) {}
  virtual inline ~vector_rep () { if (n!=0) delete[] a; }
  friend class vector<T,K>;
};

template<typename T, typename K=vector_dense>
class vector {
INDIRECT_PROTO_2 (vector, vector_rep, T, K)
public:
  vector () {
    int n= K::def_len;
    if (n == 0) rep= new vector_rep<T,K> (NULL, 0);
    else rep= new vector_rep<T,K> (new T[n], n);
  }
  vector (T x, int n=K::def_len) {
    int i;
    if (n == 0) rep= new vector_rep<T,K> (NULL, 0);
    else rep= new vector_rep<T,K> (new T[n], n);
    for (i=0; i<n; i++)
      rep->a[i]= x;
  }
  vector (T* a, int n=K::def_len) {
    rep= new vector_rep<T,K> (a, n);
  }
  T& operator[] (int i) { return rep->a[i]; }
  T operator[] (int i) const { return rep->a[i]; }
};
INDIRECT_IMPL_2 (vector, vector_rep, typename, T, typename, K)

TMPL int dim (const vector<T,K>& v) { return v->n; }

/******************************************************************************
* Fixed size vectors
******************************************************************************/

template<typename T, int N>
class vector_rep<T,vector_fixed<N> > REP_STRUCT {
protected:
  T* a;
public:
  inline vector_rep (T* a2, int dummy): a (a2) { (void) dummy; }
  virtual inline ~vector_rep () { delete[] a; }
  friend class vector<T,vector_fixed<N> >;
};

template<typename T, int N> int
dim (const vector<T,vector_fixed<N> >& v) { return N; }

/******************************************************************************
* Routines on vectors
******************************************************************************/

TMPL ostream&
operator << (ostream& out, const vector<T,K>& v) {
  int i, n= dim(v);
  out << "[ ";
  for (i=0; i<n; i++) {
    if (i != 0) out << ", ";
    out << v[i];
  }
  return out << "]";
}

/*
vector<T,K> operator - (const vector<T,K>& x);
vector<T,K> operator + (const vector<T,K>& x, const vector<T,K>& y);
vector<T,K> operator - (const vector<T,K>& x, const vector<T,K>& y);
vector<T,K> operator * (const vector<T,K>& x, const vector<T,K>& y);
vector<T,K> operator / (const vector<T,K>& x, const vector<T,K>& y);
vector<T,K> operator << (const vector<T,K>& x, mp_exp_t shift);
vector<T,K> operator >> (const vector<T,K>& x, mp_exp_t shift);
vector<T,K> abs (const vector<T,K>& x);

inline vector<T,K>& operator += (vector<T,K>& x, const vector<T,K>& y) { return x= x+y; }
inline vector<T,K>& operator -= (vector<T,K>& x, const vector<T,K>& y) { return x= x-y; }
inline vector<T,K>& operator *= (vector<T,K>& x, const vector<T,K>& y) { return x= x*y; }
inline vector<T,K>& operator /= (vector<T,K>& x, const vector<T,K>& y) { return x= x/y; }
inline vector<T,K>& operator <<= (vector<T,K>& x, mp_exp_t sh) { return x= x<<sh; }
inline vector<T,K>& operator >>= (vector<T,K>& x, mp_exp_t sh) { return x= x>>sh; }

int sign (const vector<T,K>& x, int max_cancel= MAX_CANCEL_TEST);
inline bool operator == (const vector<T,K>& x, const vector<T,K>& y) {
  return sign (x-y) == 0; }
inline bool operator != (const vector<T,K>& x, const vector<T,K>& y) {
  return sign (x-y) != 0; }
inline bool operator < (const vector<T,K>& x, const vector<T,K>& y) {
  return sign (x-y) < 0; }
inline bool operator > (const vector<T,K>& x, const vector<T,K>& y) {
  return sign (x-y) > 0; }
inline bool operator <= (const vector<T,K>& x, const vector<T,K>& y) {
  return sign (x-y) <= 0; }
inline bool operator >= (const vector<T,K>& x, const vector<T,K>& y) {
  return sign (x-y) >= 0; }
*/

#undef TMPL
}
#endif // __MML_VECTOR_HPP
